/* ************************************************************************** */
// File       : d3dvertexbuffer.cpp
// Programmer : Tobin Schwaiger-Hastanan
// Copyright (c) 1997-2002 Tobin Schwaiger-Hastanan
// Notes      : for flipcode.com's COTD.
/* ************************************************************************** */
#include <string>
using namespace std;

#include <windows.h>
#include <d3d8.h>

#include "d3dvertexbuffer.h"

CD3DVertexBuffer::CD3DVertexBuffer( LPDIRECT3DDEVICE8 pD3DDevice, int count, unsigned long format, bool isDynamic )
{
    unsigned long size = 0;
    
    m_pD3DDevice       = pD3DDevice;
    m_pD3DVertexBuffer = NULL;
    m_primitiveType    = D3DPT_POINTLIST;

    m_FVF        = 0;
    m_format     = format;
	m_nMaxVertex = count;

    if( format & FORMAT_XYZ )
    {
        m_FVF |= D3DFVF_XYZ;
        size += sizeof( float ) * 3;
        m_offsetVertex = 0;
    }
    else if( format & FORMAT_XYZRHW )
    {
        m_FVF |= D3DFVF_XYZRHW;
        size += sizeof( float ) * 4;
        m_offsetVertex = 0;
    }

    if( format & FORMAT_NORMAL )
    {     
        m_FVF |= D3DFVF_NORMAL;
        m_offsetNormal = size;
        size += sizeof( float ) * 3;        
    }

    if( format & FORMAT_DIFFUSE )
    {
        m_FVF |= D3DFVF_DIFFUSE;
        m_offsetDiffuse = size;
        size += sizeof( DWORD );
    }

    if( format & FORMAT_SPECULAR )
    {
        m_FVF |= D3DFVF_SPECULAR;
        m_offsetSpecular = size;
        size += sizeof( DWORD );
    }

    if( format & FORMAT_TEXVERTEXFLAG )
    {
        // fix this
        int nTexVertex = (format & ~FORMAT_TEXVERTEXFLAG ) >> 8;
        
        switch( nTexVertex )
        {
        case 0:
            m_FVF |= D3DFVF_TEX0;
            break;
        case 1:
            m_FVF |= D3DFVF_TEX1;            
            break;
        case 2:
            m_FVF |= D3DFVF_TEX2;            
            break;
        case 3:
            m_FVF |= D3DFVF_TEX3;            
            break;
        case 4:
            m_FVF |= D3DFVF_TEX4;            
            break;
        case 5:
            m_FVF |= D3DFVF_TEX5;            
            break;
        case 6:
            m_FVF |= D3DFVF_TEX6;            
            break;
        case 7:
            m_FVF |= D3DFVF_TEX7;            
            break;
        case 8:
            m_FVF |= D3DFVF_TEX8;            
            break;
        default:
            throw CException( __FILE__, __LINE__, "CD3DVertexBuffer::CD3DVertexBuffer() --> Number of texture vertex not supported." );
            break;
        }

        m_offsetTexVertex = size;
        m_strideTexVertex = sizeof( float ) * 2;

        size += m_strideTexVertex * nTexVertex;        
    }
    
    m_strideVertex = size;

    DWORD flags = isDynamic?D3DUSAGE_DYNAMIC:0;
    unsigned long rc;    
    if( (rc = m_pD3DDevice->CreateVertexBuffer( m_strideVertex * count, D3DUSAGE_WRITEONLY|flags , m_FVF, D3DPOOL_DEFAULT, &m_pD3DVertexBuffer ) ) != D3D_OK )
        throw CException( __FILE__, __LINE__, "CD3DVertexBuffer::CD3DVertexBuffer() --> Unable to create D3D Vertex Buffer." );

    m_isLocked = false;
}

CD3DVertexBuffer::~CD3DVertexBuffer()
{
    if( m_pD3DVertexBuffer != NULL )
        m_pD3DVertexBuffer->Release();
}

void CD3DVertexBuffer::setPrimitive( PRIMITIVE& primitive )
{
    switch( primitive )
    {
    case PRIMITIVE_POINTLIST:
        m_primitiveType = D3DPT_POINTLIST;
        break;
    case PRIMITIVE_LINELIST:
        m_primitiveType = D3DPT_LINELIST;
        break;
    case PRIMITIVE_LINESTRIP:
        m_primitiveType = D3DPT_LINESTRIP;
        break;
    case PRIMITIVE_TRIANGLELIST:
        m_primitiveType = D3DPT_TRIANGLELIST;
        break;
    case PRIMITIVE_TRIANGLESTRIP:
        m_primitiveType = D3DPT_TRIANGLESTRIP;
        break;
    case PRIMITIVE_TRIANGLEFAN:
        m_primitiveType = D3DPT_TRIANGLEFAN;
        break;
    }
}

CVertexBuffer::PRIMITIVE CD3DVertexBuffer::getPrimitive()
{
    PRIMITIVE primitive;

    switch( m_primitiveType )
    {
    case D3DPT_POINTLIST:
        primitive = PRIMITIVE_POINTLIST;
        break;
    case D3DPT_LINELIST:
        primitive = PRIMITIVE_LINELIST;
        break;
    case D3DPT_LINESTRIP:
        primitive = PRIMITIVE_LINESTRIP;
        break;
    case D3DPT_TRIANGLELIST:
        primitive = PRIMITIVE_TRIANGLELIST;
        break;
    case D3DPT_TRIANGLESTRIP:
        primitive = PRIMITIVE_TRIANGLESTRIP;
        break;
    case D3DPT_TRIANGLEFAN:
        primitive = PRIMITIVE_TRIANGLEFAN;
        break;
    }

    return primitive;
}

CVertexBuffer& CD3DVertexBuffer::begin( PRIMITIVE primitive )
{
    setPrimitive( primitive );

    m_pvertex = (unsigned char*) lock().getVertexData();
    m_nTexVertex = 0;

	return *this;
}

CVertexBuffer& CD3DVertexBuffer::end()
{
    unlock();
	return *this;
}

CVertexBuffer& CD3DVertexBuffer::lock()
{
    if( m_pD3DVertexBuffer->Lock( 0, 0, (BYTE**) &m_pbuffer, D3DLOCK_DISCARD ) != D3D_OK )
        throw CException( __FILE__, __LINE__, "CD3DVertexBuffer::lock() --> Unable to lock D3D Vertex Buffer." );

    m_isLocked = true;
    return *this;
}

CVertexBuffer& CD3DVertexBuffer::unlock()
{
    if( m_pD3DVertexBuffer->Unlock() != D3D_OK )
        throw CException( __FILE__, __LINE__, "CD3DVertexBuffer::unlock() --> Unable to unlock D3D Vertex Buffer." );

    m_isLocked = false;
    m_pbuffer = NULL;
    return *this;
}

bool CD3DVertexBuffer::isLocked()
{
    return m_isLocked;
}

void* CD3DVertexBuffer::getVertexData()
{
    return m_pbuffer;
}

int CD3DVertexBuffer::getVertexCount()
{
    return m_nVertex;
}

int CD3DVertexBuffer::getMaxVertexCount()
{
    return m_nMaxVertex;
}

unsigned long CD3DVertexBuffer::getVertexFormat()
{
    return m_format;
}

void CD3DVertexBuffer::vertex( float x, float y, float z )
{
    float* vertex = (float*)(m_pvertex + m_offsetVertex);

    vertex[ 0 ] = x;
    vertex[ 1 ] = y;
    vertex[ 2 ] = z;

    m_nTexVertex = 0;
    m_pvertex += m_strideVertex;
    m_nVertex ++;
}

void CD3DVertexBuffer::vertex( float x, float y, float z, float w )
{
    float* vertex = (float*)(m_pvertex + m_offsetVertex);

    vertex[ 0 ] = x;
    vertex[ 1 ] = y;
    vertex[ 2 ] = z;
    vertex[ 3 ] = w;

    m_nTexVertex = 0;
    m_pvertex += m_strideVertex;
    m_nVertex ++;
}
void CD3DVertexBuffer::normal( float x, float y, float z )
{
    float* normal = (float*)( m_pvertex + m_offsetNormal );

    normal[ 0 ] = x;
    normal[ 1 ] = y;
    normal[ 2 ] = z;
}

void CD3DVertexBuffer::diffuse( float r, float g, float b, float a  )
{
    DWORD* diffuse = (DWORD*)( m_pvertex + m_offsetDiffuse );
    
    *diffuse = D3DCOLOR_COLORVALUE( r, g, b, a );
}

void CD3DVertexBuffer::specular( float r, float g, float b, float a  )
{
    DWORD* specular = (DWORD*)( m_pvertex + m_offsetSpecular );
    
    *specular = D3DCOLOR_COLORVALUE( r, g, b, a );
}

void CD3DVertexBuffer::texvertex( float u, float v )
{
    float* texvertex = (float*)( m_pvertex + m_offsetTexVertex );

    texvertex += m_strideTexVertex * m_nTexVertex;
    texvertex[ 0 ] = u;
    texvertex[ 1 ] = v;
    m_nTexVertex ++;    
}