/* ************************************************************************** */
// File       : d3dvertexbuffer.h
// Programmer : Tobin Schwaiger-Hastanan
// Copyright (c) 1997-2002 Tobin Schwaiger-Hastanan
// Notes      : for flipcode.com's COTD.
/* ************************************************************************** */
#ifndef _D3DVERTEXBUFFER_DAWT_H_INCLUDED_
#define _D3DVERTEXBUFFER_DAWT_H_INCLUDED_

/**
  * This is Direct 3D 8.1 implementation of CVertexBuffer.
  *
  * @see    CVertexBuffer
  * @author Tobin Schwaiger-Hastanan
  */
class CD3DVertexBuffer:public CVertexBuffer
{
protected:    
    CD3DVertexBuffer();                          // disable default constructor
    CD3DVertexBuffer( const CD3DVertexBuffer& ); // disable copy constructor    
    CD3DVertexBuffer& operator =( const CD3DVertexBuffer& ); // disable assignment operator
public:
    /**
      * Constructor used to create vertex buffer.
      * @param pD3DDevice Direct 3D Device
      * @param count number of vertex to be created
      * @param format format of vertex buffer ( See format flags in CVertexBuffer)
      * @param isDynamic specifies that vertex buffer is dynamic. [default: true]
      */
    CD3DVertexBuffer( LPDIRECT3DDEVICE8 pD3DDevice, int count, unsigned long format,
                      bool isDynamic = true );

    // destructor
    ~CD3DVertexBuffer();
    
    // for the following methods, see CVertexBuffer
    virtual CVertexBuffer& lock();
    virtual CVertexBuffer& unlock();
    virtual bool isLocked();

    virtual void setPrimitive( PRIMITIVE& primitive );
    virtual PRIMITIVE getPrimitive();

    virtual CVertexBuffer& begin( PRIMITIVE primitive = PRIMITIVE_POINTLIST );
    virtual CVertexBuffer& end();

    virtual void* getVertexData();
    
    virtual int getVertexCount();
    virtual int getMaxVertexCount();
    virtual unsigned long getVertexFormat();
    virtual unsigned long getVertexStride() { return m_strideVertex; }

    virtual CVertexBuffer& vertex( float x, float y, float z );
    virtual CVertexBuffer& vertex( float x, float y, float z, float w );
    virtual CVertexBuffer& normal( float x, float y, float z );
    virtual CVertexBuffer& diffuse( float r, float g, float b, float a = 1.0f );
    virtual CVertexBuffer& specular( float r, float g, float b, float a = 1.0f );
    virtual CVertexBuffer& texvertex( float u, float v );
    
    /* d3d related methods */
    /**
      * Returns the D3D Primitive type. 
      * This is a CD3DVertexBuffer specific method.
      *
      * @return Direct 3D primitive type.
      */
    D3DPRIMITIVETYPE        getD3DPrimitiveType() { return m_primitiveType;    }
    
    /**
      * Returns the D3D Vertex buffer created by this class. 
      * This is a CD3DVertexBuffer specific method.
      *
      * @return pointer to a Direct 3D Vertex Buffer.
      */
    LPDIRECT3DVERTEXBUFFER8 getD3DVertexBuffer()  { return m_pD3DVertexBuffer; }
    /**
      * Returns the D3D Vertex buffer format. 
      * This is a CD3DVertexBuffer specific method.
      *
      * @return pointer to a Direct 3D Vertex Buffer Format.
      */
    DWORD                   getD3DVertexFormat()  { return m_FVF; }

private:

    bool          m_isLocked;   // flag to specify if buffer is locked        
    int           m_nMaxVertex; // maximum vertex in buffer.
    int           m_nVertex;    // number of vertex currently in buffer
    unsigned long m_format;     // vertex format    
    void*         m_pbuffer;    // pointer to vertex data

    unsigned char* m_pvertex;    // pointer to head of current vertex

    
    unsigned long m_offsetVertex;   // offset of vertex data start of vertex
    unsigned long m_strideVertex;   // stride of entire vertex data
    unsigned long m_offsetNormal;   // offset of normal data from start of vertex
    unsigned long m_offsetDiffuse;  // offset of diffuse data from start of vertex
    unsigned long m_offsetSpecular; // offset of specular data from start of vertex
    unsigned long m_offsetTexVertex; // offset of TexVertex data from start of vertex
    unsigned long m_strideTexVertex; // stride of a single texture vertex entry
    int m_nTexVertex; // number of texture vertex
    
    
    /* d3d related members */
    D3DPRIMITIVETYPE        m_primitiveType;    // d3d primitive type
    LPDIRECT3DVERTEXBUFFER8 m_pD3DVertexBuffer; // d3d vertex buffer
    LPDIRECT3DDEVICE8       m_pD3DDevice;        // d3d device
    DWORD                   m_FVF;                // d3d vertex format
};
#endif